include std/datetime.e
include std/math.e

-- Date Calculations

-- Sep 1, 1996
-- written by Junko C. Miura, Rapid Deployment Software
-- Contributed to the Public Domain

-- Aug 5, 2010
-- converted by Jean-Marc Duro to a reusable unit

-- procedures and functions definition
function INT(atom x)
  if x >= 0 then
    return floor(x)
  else
    return -1 * floor(-x)
  end if
end function

public function getFactor(integer yyyy, integer mm, integer dd)
  integer factor
     
  if yyyy <= 1582 then
	puts(2, "year must be > 1582\n")
	return -1
  end if
  if mm < 1 or mm > 12 then
	puts(2, "month must be > 0 and <= 12\n")
	return -1
  end if
  if mm < 1 or mm > 31 then
	puts(2, "day must be > 0 and <= 31\n")
	return -1
  end if
  if mm > 2 then
    factor = 365 * yyyy + dd + 31 * (mm - 1) - INT(0.4 * mm + 2.3)
		 + INT(yyyy / 4) - INT(0.75 * (INT(yyyy / 100) + 1))
  else
	factor = 365 * yyyy + dd + 31 * (mm - 1) + INT((yyyy - 1) / 4)
		 - INT(0.75 * (INT(((yyyy - 1) / 100) + 1)))
  end if
  return factor
end function

public function dayOfWeek(integer yyyy, integer mm, integer dd)
  integer factor, dayOfWeek
    
  factor = getFactor(yyyy, mm, dd)
  dayOfWeek = factor + (INT(-factor / 7) * 7)
  if dayOfWeek > 6 then              -- start of DEBUG
	printf(2, "WARNING-- dayOfWeek : %d\n", dayOfWeek)
	dayOfWeek = remainder(dayOfWeek, 7) 
  end if                             -- end of DEBUG
  return dayOfWeek + 1
end function

public function getDaysBetwDates(sequence date1, sequence date2)
  integer factor1, factor2, numDays
    
  factor1 = getFactor(date1[1],date1[2],date1[3])
  factor2 = getFactor(date2[1],date2[2],date2[3])
  if factor2 > factor1 then
    numDays = factor2 - factor1
  else
    numDays = factor1 - factor2
  end if
  return numDays
end function

public function excelToDatetime(atom exc)
  datetime d = new(1899, 12, 30, 0, 0, 0)
  atom t = frac(exc) * 24
  integer hr = trunc(t)
  t = (t-hr) * 60
  integer mn = trunc(t)
  t = (t-mn) * 60
  integer ss = trunc(t)
  d = add(d, trunc(exc), DAYS)
  d = add(d, hr, HOURS)
  d = add(d, mn, MINUTES)
  d = add(d, ss, SECONDS)
  return d 
end function

public function datetimeToExcel(sequence dt)
  atom nb = getDaysBetwDates({1899,12,30}, {dt[YEAR],dt[MONTH],dt[DAY]})
  nb += (dt[SECOND]+dt[MINUTE]*60+dt[HOUR]*3600)/86400
  return nb 
end function

-- 41914.72292
-- 02/10/2014 17:21

-- 42250.48472
-- 03/09/2015 11:38

